package org.openscm.kundo.plugins

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.openscm.kundo.plugins.context.BuildContext

/**
 * Simple Checkstyle Delegate
 * @author <a href="mailto:steve.king@upco.co.uk">Steve king</a>
 * @version 1.0.0
 * 
 * Description: A Checkstyle Delegate that cretaes Checkstyle reports
 * 
 * <p>The Checkstyle plugin cretaes checkstyle reports on coding standards</p>
 * 
 * 
 * <p>These properties are automatically injected into the plugin at runtime.
 * This is done by the exposing of setter methods for variables mimicking the
 * name of the . separated ant property but replacing the dots with uppercase
 * letters resulting in a camelCase variable name.</p>
 * 
 * @see 	#setEnabled( String )
 * @see 	#setReportdir( String )
 * @see 	#setReportname( String )
 * @see 	#setConfig( String )
 * @see 	#setAllowTestdir( String ) 
 * @see 	#setFailonviolation( String ) 
 */
/* 
 * @ant-target doCheckstyle
 */
class CheckstyleDelegate extends AbstractPluginTargetDelegate {

	/*
	 * A flag if checkstyle reporting is enabled
	 * @ant-property-name checkstyle.enabled
	 */
	String enabled
	/*
	 * The directory to have the checkstyle report writtenm to
	 * @ant-property-name checkstyle.reportdir
	 */
	String reportdir
	/*
	 * ~The name to assign the checkstyle report
	 * @ant-property-name checkstyle.allow.testdir
	 */
	String reportname	
	/*
	 * The location of the checkstyle config_config.xml file
	 * @ant-property-name checkstyle.config
	 */	 
	String config
	/*
	 * A flag to allow test directorys
	 * @ant-property-name checkstyle.allow.testdir
	 */
	String allowTestdir
	/*
	 * A flag to determine if a build should fail upon encountering a checkstyle violation
	 * @ant-property-name checkstyle.failonviolation
	 */
	String failonviolation

	/**
	 * Constructor sets ant and buildContext instances in super class
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	CheckstyleDelegate( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext)
	}
	
	/**
	 * Setter method for member variable enabled
	 * @param enabled	String representation of a flag to enable checkstyle reporting
	 */
	void setEnabled( String enabled ){
		this.enabled = enabled
	}
	
	/**
	 * Setter method for member variable reportdir
	 * @param reportdir	String representation of the directory path to write reports to
	 */	
	void setReportdir( String reportdir ){
		this.reportdir = reportdir
	}
	
	/**
	 * Setter method for member variable reportname
	 * @param reportname	String representation of report name
	 */	
	void setReportname( String reportname ){
		this.reportname = reportname
	}
	/**
	 * Setter method for member variable enabled
	 * @param config	String representation of the checkstyle_config.xml file location
	 */
	void setConfig( String config ){
		this.config = config
	}
	
	/**
	 * Setter method for member variable reportdir
	 * @param allowTestdir	String representation ....
	 */	
	void setAllowTestdir( String allowTestdir ){
		this.allowTestdir = allowTestdir
	}
	
	/**
	 * Setter method for member variable failonviolation
	 * @param failonviolation String representation ....
	 */	
	void setFailonviolation( String failonviolation ){
		this.failonviolation = failonviolation
	}
	
	/**
	 * Private method for executing the checkstyle report
	 */
	private void checkstyle() {
		
		 // retrieve project plugins
		Map properties = buildContext.get( "project.properties" )
		List srcProjectProperties = properties.get( "src" )
		List testProjectProperties = properties.get( "test" )
		
		ant.taskdef(resource:"checkstyletask.properties", classpathref:"kundo-checkstyle-plugin.dependency.path")
		ant.mkdir(dir:reportdir)
		 
		ant.checkstyle(config:config, failOnViolation:failonviolation){
			srcProjectProperties.each{srcProjectProperty ->
				ant.fileset( dir:srcProjectProperty.output, includes:srcProjectProperty.includes )
			}
			//check if test clases need reporting on
			if (allowTestdir.equalsIgnoreCase("true"))
				testProjectProperties.each{testProjectProperty ->
				ant.fileset( dir:testProjectProperty.output, includes:testProjectProperty.includes )
			}
			ant.formatter(type:"xml", toFile:reportdir + reportname)
		}
	}
	
	/**
	 * Implementation of abstract method from AbstractPluginTargetDelegate
	 * to hold assertions that all global and local properties are available to the plugin.
	 * This method is automatically called prior to execution.
	 */
	void doCheck(){
		assert enabled != null
		assert reportdir != null
		assert config != null
		assert allowTestdir != null
		assert buildContext.get( "project.properties" ) != null
		assert (enabled.equalsIgnoreCase("true") || enabled.equalsIgnoreCase("false"))
		assert (allowTestdir.equalsIgnoreCase("true") || allowTestdir.equalsIgnoreCase("false"))
	}
	
	/**
	 * Implementation of abstract method from AbstractPluginTargetDelegate
	 * this method is automatically called.
	 */	
	void doExecute(){		
		
		// check if plugin is enabled
		if ( log.isDebugEnabled() )
			log.debug( "Begin Checkstyle plugin")
		if (enabled.equalsIgnoreCase("true"))
			checkstyle()
	}
	
}